@@ -463,7 +463,7 @@ def ensure_tree_conflict(sbox, operation,
       if operation == 'update':
         logger.debug("--- Updating")
         run_and_verify_svn(expected_stdout, [],
-                           'update', target_path)
+                           'update', target_path, '--accept=postpone')
       elif operation == 'switch':
         logger.debug("--- Switching")
         run_and_verify_svn(expected_stdout, [],
@@ -1339,7 +1339,7 @@ def actual_only_node_behaviour(sbox):
                      "relocate", A_copy_url + "/foo", foo_path)
 
   # resolve
-  expected_stdout = "Resolved conflicted state of.*foo.*"
+  expected_stdout = "Tree conflict at.*foo.*marked as resolved"
   expected_stderr = []
   run_and_verify_svn(expected_stdout, expected_stderr,
                      "resolve", "--accept", "working", foo_path)
